@@ -156,21 +156,22 @@ static RD_UNUSED const char *rd_kafka_ApiKey2str(int16_t ApiKey) {
                 "DescribeUserScramCredentialsRequest",
             [RD_KAFKAP_AlterUserScramCredentials] =
                 "AlterUserScramCredentialsRequest",
-            [RD_KAFKAP_Vote]                 = "VoteRequest",
-            [RD_KAFKAP_BeginQuorumEpoch]     = "BeginQuorumEpochRequest",
-            [RD_KAFKAP_EndQuorumEpoch]       = "EndQuorumEpochRequest",
-            [RD_KAFKAP_DescribeQuorum]       = "DescribeQuorumRequest",
-            [RD_KAFKAP_AlterIsr]             = "AlterIsrRequest",
-            [RD_KAFKAP_UpdateFeatures]       = "UpdateFeaturesRequest",
-            [RD_KAFKAP_Envelope]             = "EnvelopeRequest",
-            [RD_KAFKAP_FetchSnapshot]        = "FetchSnapshot",
-            [RD_KAFKAP_DescribeCluster]      = "DescribeCluster",
-            [RD_KAFKAP_DescribeProducers]    = "DescribeProducers",
-            [RD_KAFKAP_BrokerHeartbeat]      = "BrokerHeartbeat",
-            [RD_KAFKAP_UnregisterBroker]     = "UnregisterBroker",
-            [RD_KAFKAP_DescribeTransactions] = "DescribeTransactions",
-            [RD_KAFKAP_ListTransactions]     = "ListTransactions",
-            [RD_KAFKAP_AllocateProducerIds]  = "AllocateProducerIds",
+            [RD_KAFKAP_Vote]                   = "VoteRequest",
+            [RD_KAFKAP_BeginQuorumEpoch]       = "BeginQuorumEpochRequest",
+            [RD_KAFKAP_EndQuorumEpoch]         = "EndQuorumEpochRequest",
+            [RD_KAFKAP_DescribeQuorum]         = "DescribeQuorumRequest",
+            [RD_KAFKAP_AlterIsr]               = "AlterIsrRequest",
+            [RD_KAFKAP_UpdateFeatures]         = "UpdateFeaturesRequest",
+            [RD_KAFKAP_Envelope]               = "EnvelopeRequest",
+            [RD_KAFKAP_FetchSnapshot]          = "FetchSnapshot",
+            [RD_KAFKAP_DescribeCluster]        = "DescribeCluster",
+            [RD_KAFKAP_DescribeProducers]      = "DescribeProducers",
+            [RD_KAFKAP_BrokerHeartbeat]        = "BrokerHeartbeat",
+            [RD_KAFKAP_UnregisterBroker]       = "UnregisterBroker",
+            [RD_KAFKAP_DescribeTransactions]   = "DescribeTransactions",
+            [RD_KAFKAP_ListTransactions]       = "ListTransactions",
+            [RD_KAFKAP_AllocateProducerIds]    = "AllocateProducerIds",
+            [RD_KAFKAP_ConsumerGroupHeartbeat] = "ConsumerGroupHeartbeat",
         };
         static RD_TLS char ret[64];
 
@@ -584,10 +585,60 @@ typedef struct rd_kafka_Uuid_s {
 } rd_kafka_Uuid_t;
 
 #define RD_KAFKA_UUID_ZERO                                                     \
-        { 0, 0, "" }
+        (rd_kafka_Uuid_t) {                                                    \
+                0, 0, ""                                                       \
+        }
+
+#define RD_KAFKA_UUID_IS_ZERO(uuid)                                            \
+        (!rd_kafka_Uuid_cmp(uuid, RD_KAFKA_UUID_ZERO))
 
 #define RD_KAFKA_UUID_METADATA_TOPIC_ID                                        \
-        { 0, 1, "" }
+        (rd_kafka_Uuid_t) {                                                    \
+                0, 1, ""                                                       \
+        }
+
+static RD_INLINE RD_UNUSED int rd_kafka_Uuid_cmp(rd_kafka_Uuid_t a,
+                                                 rd_kafka_Uuid_t b) {
+        if (a.most_significant_bits < b.most_significant_bits)
+                return -1;
+        if (a.most_significant_bits > b.most_significant_bits)
+                return 1;
+        if (a.least_significant_bits < b.least_significant_bits)
+                return -1;
+        if (a.least_significant_bits > b.least_significant_bits)
+                return 1;
+        return 0;
+}
+
+static RD_INLINE RD_UNUSED int rd_kafka_Uuid_ptr_cmp(void *a, void *b) {
+        rd_kafka_Uuid_t *a_uuid = a, *b_uuid = b;
+        return rd_kafka_Uuid_cmp(*a_uuid, *b_uuid);
+}
+
+rd_kafka_Uuid_t rd_kafka_Uuid_random();
+
+const char *rd_kafka_Uuid_str(const rd_kafka_Uuid_t *uuid);
+
+unsigned int rd_kafka_Uuid_hash(const rd_kafka_Uuid_t *uuid);
+
+unsigned int rd_kafka_Uuid_map_hash(const void *key);
+
+/**
+ * @brief UUID copier for rd_list_copy()
+ */
+static RD_UNUSED void *rd_list_Uuid_copy(const void *elem, void *opaque) {
+        return (void *)rd_kafka_Uuid_copy((rd_kafka_Uuid_t *)elem);
+}
+
+static RD_INLINE RD_UNUSED void rd_list_Uuid_destroy(void *uuid) {
+        rd_kafka_Uuid_destroy((rd_kafka_Uuid_t *)uuid);
+}
+
+static RD_INLINE RD_UNUSED int rd_list_Uuid_cmp(const void *uuid1,
+                                                const void *uuid2) {
+        return rd_kafka_Uuid_cmp(*((rd_kafka_Uuid_t *)uuid1),
+                                 *((rd_kafka_Uuid_t *)uuid2));
+}
 
 
 /**
